package keel.Algorithms.FingerprintClassification.Liu10;

import java.util.Arrays;

public class Node {

	private static int DATA_SIZE;
	private boolean expanded;
	private int leftChild;
	private int rightChild;
	private int parent;
	private boolean leaf;
	
	private int splitAttribute;
	private double splitValue;
	private double splitGini;
	
	private boolean data [];
	
	private double positives;
	private double negatives;
	private int count;
	private double countWeights;
	
	private double logLikehood;
	
	public static void setDataSize(int value){
		DATA_SIZE=value;
	}
	
	public Node(){
		
		expanded = false;
		leftChild = -1;
		rightChild = -1;
		parent = -1;
		leaf = false;
		data = new boolean [DATA_SIZE];
		
		Arrays.fill(data, false);
		
		positives = 0.0;
		negatives = 0.0;
		countWeights=0.0;
		count=0;
	}
	
	public void setParent(int value){
		parent=value;
	}
	
	public int getParent(){
		return parent;
	}
	
	public void setLeftChildren(int value){
		leftChild=value;
	}
	
	public int getLeftChildren(){
		return leftChild;
	}
	
	public void setRightChildren(int value){
		rightChild=value;
	}
	
	public int getRightChildren(){
		return rightChild;
	}
	
	public void setAllData(double weights []){
		
		Arrays.fill(data, true);
		count=0;
		countWeights=0.0;
		
		for(int i=0;i<weights.length;i++){
			count+=1;
			countWeights+=weights[i];
		}
		
	}
	
	public void setDataArray(boolean array [],double weights []){
		
		count=0;
		countWeights=0.0;
		
		for(int i=0;i<array.length;i++){
			data[i]=array[i];
			if(data[i]){
				count+=1;
				countWeights+=weights[i];
			}
		}
		
	}
	
	public boolean [] accessDataArray(){
		
		return data;
	}
	
	public void setData(int index,double weights []){
		
		if(!data[index]){
			countWeights+=weights[index];
			count+=1;
		}
		data[index] = true;
		
	}
	
	public void unSetData(int index,double weights []){
		
		if(data[index]){
			countWeights-=weights[index];
			count-=1;
		}
		
		data[index] = false;
		
	}
	
	public int getCount(){
		return count;
	}
	
	public double getWeightCount(){
		return countWeights;
	}
	
	public boolean contains(int index){
		return data[index];
	}
	
	public boolean isExpanded(){
		return expanded;
	}
	
	public void setExpanded(){
		expanded=true;
	}
	
	public void resetPositives(){		
		positives = 0.0;		
	}
	
	public void resetNegatives(){		
		negatives = 0.0;		
	}
	
	public void addPositive(double weight){		
		positives+=weight;		
	}
	
	public void addNegative(double weight){		
		negatives+=weight;	
	}
	
	public double getPositives(){		
		return positives;		
	}
	
	public double getNegatives(){		
		return negatives;		
	}
	
	public double getProbPositives(){			
		return positives/(positives+negatives);		
	}
	
	public double getProbNegatives(){		
		return negatives/(positives+negatives);		
	}
	
	public double getGiniIndex(){		
		
		double gini=1.0;
		double value,value2;
		
		value=getProbPositives();
		value*=value;
		
		value2=getProbNegatives();
		value2*=value2;
		
		gini-=value;
		gini-=value2;
		
		return gini;
		
	}
	
	public void setBestSplit(double bestGini,int bestSplitAttribute,double bestSplitValue){
		
		splitAttribute=bestSplitAttribute;
		splitValue=bestSplitValue;
		splitGini=bestGini;
	}
	
	public double getBestGini(){
		return splitGini;
	}
	
	public double getSplitValue(){
		return splitValue;
	}
	
	public int getSplitAttribute(){
		return splitAttribute;
	}
	
	public void setLeaf(){
		
		leaf=true;
	}
	
	public boolean isLeaf(){	
		return leaf;
	}
	
	public void setLogLikehood(double value){
		logLikehood=value;
	}
	
	public double getLogLikehood(){
		return logLikehood;
	}
	
	public int propagateInstance(double instance []){

		if(leaf){
			return 0;
		}
		
		if(instance[splitAttribute]<=splitValue){
			return leftChild;
		}
		else{
			return rightChild;
		}
	}
}
