package keel.Algorithms.FingerprintClassification.Liu10;

import java.util.Arrays;

public class Tree {

	private static double data [][];
	private static Pair sortData [][];
	private static int LEAF_LIMIT;
	
	private final static double EPSILON = 0.000001;
	
	private double weights [];
	private int output [];
	private int key;
	private Node body[];
	private int size;
	private int actual_size;
	
	public static void setData(double newData [][]){
		
		data= new double [newData.length][newData[0].length];
		
		for(int i=0;i<newData.length;i++){
			System.arraycopy(newData[i], 0, data[i], 0, newData[0].length);
		}
		
		sortData();
	}
	
	private static void sortData(){
		
		sortData = new Pair [data[0].length][data.length];
		
		//generate indexes
		for(int j=0;j<data[0].length;j++){
			for(int i=0;i<data.length;i++){
				sortData[j][i]= new Pair (i,data[i][j]);
			}
		}
		
		//sort indexes
		for(int j=0;j<data[0].length;j++){
			Arrays.sort(sortData[j]);
		}

	}
	
	public static void setLeafsLimit(int value){
		LEAF_LIMIT=value;
	}
	
	public Tree(int value, double newWeights [],int newOutput[]){
		
		key=value;
		
		weights = new double [newWeights.length];
		System.arraycopy(newWeights, 0, weights, 0, newWeights.length);
		
		output = new int [newOutput.length];
		System.arraycopy(newOutput, 0, output, 0, newOutput.length);
	}
		
	public int getKey(){
		
		return key;
	}
	
	public void generateRoot(){
		
		size = (2 * LEAF_LIMIT) -1; 
		body = new Node[size];
		
		
		for(int i=0;i< body.length;i++){
			body[i]=new Node();
		}
		
		actual_size=1;
		body[0].setAllData(weights);
		
		evaluateNode(0);
	}
	
	public boolean isFinished(){
		
		return body[size-1].isExpanded();
	}
	
	public void evaluateNode(int index){
		
		boolean terminal=false;
		
		int bestSplitAttribute=0;
		double bestSplitValue=0.0;
		double bestGini=1.0;
		
		double pos1,pos2,neg1,neg2;
		double gini,gini1,gini2,fraction;
		
		double sumWeights, totalWeights;
		
		//compute node statistics
		body[index].resetNegatives();
		body[index].resetPositives();

		for(int i=0;i<data.length;i++){
			if(body[index].contains(i)){
				if(output[i]==-1){
					body[index].addNegative(weights[i]);

				} 
				if(output[i]==1){
					body[index].addPositive(weights[i]);
				}
			}
		}
		
		//check if the node is terminal (i.e. will not be expanded)
		if ((index == (size-1))||(body[index].getCount()<3)){
			terminal = true;
		}
		else{
			if((body[index].getProbPositives()<=0.0)||(body[index].getProbNegatives()<=0.0)){
				
				terminal = true;
			}
		}
		
		if(terminal){
			body[index].setExpanded();
			body[index].setLeaf();
		}
		else{
			//the node is a candidate to expand: Best possible split must be found
			totalWeights= body[index].getWeightCount();

			for(int j=0;j<data[0].length;j++){
				pos1=0.0; neg1=0.0;
				pos2=(double)body[index].getPositives();
				neg2=(double)body[index].getNegatives();
				gini=1.0;
				
				sumWeights=0.0;
				
				for(int i=0;i<data.length-1;i++){
					if(body[index].contains(sortData[j][i].index)){

						sumWeights+=weights[sortData[j][i].index];

						if(output[sortData[j][i].index]==1){
							pos1+=weights[sortData[j][i].index];pos2-=weights[sortData[j][i].index];
						}
						else{
							neg1+=weights[sortData[j][i].index];neg2-=weights[sortData[j][i].index];
						}
						
						//consider instances with the same attribute value
						while((i<data.length-1)&&(data[sortData[j][i+1].index][j]==data[sortData[j][i].index][j])){
							i+=1;
							if(body[index].contains(sortData[j][i].index)){
								sumWeights+=weights[sortData[j][i].index];

								if(output[sortData[j][i].index]==1){
									pos1+=weights[sortData[j][i].index];pos2-=weights[sortData[j][i].index];
								}
								else{
									neg1+=weights[sortData[j][i].index];neg2-=weights[sortData[j][i].index];
								}

							}
						}
						
						if(sumWeights<totalWeights){
							gini1=computeGiniIndex(pos1/sumWeights,neg1/sumWeights);
							gini2=computeGiniIndex(pos2/(totalWeights-sumWeights),neg2/(totalWeights-sumWeights));
							fraction=sumWeights/totalWeights;
								
							gini=(fraction*gini1)+((1.0-fraction)*gini2);
							
							if(gini<bestGini){
								bestGini=gini;
								bestSplitAttribute=j;
								bestSplitValue=data[sortData[j][i].index][j];
							}
						}

					}
					
				}
			}
			
			body[index].setBestSplit(bestGini,bestSplitAttribute,bestSplitValue);

		}
		
	}
	
	public double computeGiniIndex(double a,double b){		
		
		double gini=1.0;
		
		gini-=(a*a);
		gini-=(b*b);
		
		return gini;
		
	}
	
	public void expand(){

		double best;
		int position=-1;
		boolean dataLeft [];
		boolean dataRight [];
		boolean dataParent [];
		int countLeft,countRight;
		
		best=1.1; //greater than 1.0
		
		//search for the best possible node to expand
		for(int i=0;i<actual_size;i++){
			if((!body[i].isExpanded())&&(best>body[i].getBestGini())){
				best=body[i].getBestGini();
				position=i;
			}
		}
		
		if(position==-1){
			body[size-1].setExpanded();
			body[size-1].setLeaf();
			return;
		}
		
		//split data
		countLeft=0;
		countRight=0;
		dataLeft = new boolean [data.length];
		dataRight = new boolean [data.length];
		dataParent = new boolean [data.length];
		
		System.arraycopy(body[position].accessDataArray(), 0, dataParent, 0, data.length);
		
		for(int i=0;i<data.length;i++){
			
			if(dataParent[i]==true){
				if(data[i][body[position].getSplitAttribute()]<=body[position].getSplitValue()){
					dataLeft[i]=true;
					dataRight[i]=false;
					countLeft++;
				}
				else{
					dataLeft[i]=false;
					dataRight[i]=true;
					countRight++;
								
				}
			}
			else{
				dataLeft[i]=false;
				dataRight[i]=false;
			}
		}
		
		if((countLeft==0)||(countRight==0)){
			body[position].setExpanded();
			body[position].setLeaf();
			return;
		}
		
		//create new children
		body[actual_size].setParent(position);
		body[actual_size+1].setParent(position);
		
		body[position].setLeftChildren(actual_size);
		body[position].setRightChildren(actual_size+1);
		
		body[actual_size].setDataArray(dataLeft,weights);
		body[actual_size+1].setDataArray(dataRight,weights);
		
		//evaluate new nodes
		evaluateNode(actual_size);
		evaluateNode(actual_size+1);
		
		//update new size
		actual_size+=2;
		
		//mark current node as expanded
		body[position].setExpanded();

	}
	
	public void consolide(){
		
		double likehood;
		double fraction;
		
		//check all tree nodes, searching for leaves
		for(int i=0;i<actual_size;i++){
			if(!body[i].isExpanded()){
				body[i].setExpanded();
				body[i].setLeaf();
			}
			
			if(body[i].isLeaf()){
				
				//compute the log-likehood ratio of each leaf node
				fraction=(body[i].getPositives()+EPSILON)/(body[i].getNegatives()+EPSILON);
				likehood=0.5*Math.log(fraction);
				
				body[i].setLogLikehood(likehood);
				
			}
		}
	}
	
	public double computeInstanceLogLikehood(double instance[]){
		
		int currentNode=0;
		
		//classify the instance until reaching the leaf node
		while(!body[currentNode].isLeaf()){
			currentNode=body[currentNode].propagateInstance(instance);
		}
		
		return body[currentNode].getLogLikehood();
		
	}
	
	
}
